
#include <Bounce.h>

Bounce button0 = Bounce(6, 10);
Bounce button1 = Bounce(3, 10);  // 10 = 10 ms debounce time
Bounce button2 = Bounce(4, 10);  // which is appropriate for
Bounce button3 = Bounce(1, 10);  // most mechanical pushbuttons
Bounce button4 = Bounce(0, 10);
Bounce button5 = Bounce(2, 10);
Bounce button6 = Bounce(5, 10);

// Cursor Keys
// UP, DOWN, LEFT, RIGHT, HOME, END, PAGE UP, PAGE DOWN, TAB (9)393931I

// Normal Keys
// KEY_A	    KEY_B	    KEY_C	    KEY_D
// KEY_E	    KEY_F	    KEY_G	    KEY_H
// KEY_I	    KEY_J	    KEY_K	    KEY_L
// KEY_M	    KEY_N	    KEY_O	    KEY_P
// KEY_Q	    KEY_R	    KEY_S	    KEY_T
// KEY_U	    KEY_V	    KEY_W	    KEY_X
// KEY_Y	    KEY_Z	    KEY_1	    KEY_2
// KEY_3	    KEY_4    	    KEY_5	    KEY_6
// KEY_7	    KEY_8	    KEY_9	    KEY_0
// KEY_ENTER	    KEY_ESC	    KEY_BACKSPACE   KEY_TAB
// KEY_SPACE	    KEY_MINUS	    KEY_EQUAL	    KEY_LEFT_BRACE
// KEY_RIGHT_BRACE  KEY_BACKSLASH   KEY_NUMBER	    KEY_SEMICOLON
// KEY_QUOTE	    KEY_TILDE       KEY_COMMA	    KEY_PERIOD
// KEY_SLASH	    KEY_CAPS_LOCK   KEY_F1	    KEY_F2
// KEY_F3	    KEY_F4          KEY_F5	    KEY_F6
// KEY_F7	    KEY_F8	    KEY_F9	    KEY_F10
// KEY_F11	    KEY_F12	    KEY_PRINTSCREEN KEY_SCROLL_LOCK
// KEY_PAUSE	    KEY_INSERT	    KEY_HOME	    KEY_PAGE_UP
// KEY_DELETE	    KEY_END	    KEY_PAGE_DOWN   KEY_RIGHT
// KEY_LEFT	    KEY_DOWN	    KEY_UP	    KEY_NUM_LOCK
// KEYPAD_SLASH	    KEYPAD_ASTERIX  KEYPAD_MINUS    KEYPAD_PLUS
// KEYPAD_ENTER	    KEYPAD_1	    KEYPAD_2	    KEYPAD_3
// KEYPAD_4	    KEYPAD_5	    KEYPAD_6	    KEYPAD_7
// KEYPAD_8	    KEYPAD_9	    KEYPAD_0	    KEYPAD_PERIOD

//               00000   10000      01000  11000  00100  10100  01100          11100
byte Keys0[] = { 0,      KEY_SPACE, KEY_E, KEY_I, KEY_O, KEY_C, KEY_A,         KEY_D,  //  0..7
//               00010   10010      01010  11010  00110  10110  01110          11110
                 KEY_S,  KEY_K,     KEY_T, KEY_R, KEY_N, KEY_Y, KEY_BACKSPACE, KEY_F,  //  8..15
//               00001   10001      01000  11000  00100  10100  01100          11101
                 KEY_U,  KEY_H,     KEY_V, KEY_L, KEY_Q, KEY_Z, KEY_P,         0,      // 16..23
//               00011   10011      01011  11011  00111  10111  01111          11111
                 KEY_G,  KEY_J,     0,     KEY_W, KEY_B, KEY_X, KEY_M,         0 };    // 24..31

//               00000      10000  01000  11000  00100  10100  01100      11100
byte Keys1[] = { 0,         0,     KEY_0, 0,     KEY_1, 0,     KEY_2,     0,    //  0..7
//               00010      10010  01010  11010  00110  10110  01110      11110
                 KEY_3,     0,     KEY_4, 0,     KEY_5, 0,     KEY_6,     0,    //  8..15
//               00001      10001  01000  11000  00100  10100  01100      11101
                 KEY_7,     0,     KEY_8, 0,     KEY_9, 0,     KEY_MINUS, 0,    // 16..23
//               00011      10011  01011  11011  00111  10111  01111      11111
                 KEY_EQUAL, 0,     KEY_4, 0,     KEY_4, 0,     KEY_4,     0 };  // 24..31

//               00000      10000  01000     11000  00100          10100  01100        11100
byte Keys2[] = { 0,         0,     KEY_LEFT, 0,     KEY_UP,        0,     KEY_HOME,    0,    //  0..7
//               00010      10010  01010     11010  00110          10110  01110        11110
                 KEY_RIGHT, 0,     0,        0,     0,             0,     KEY_PAGE_UP, 0,    //  8..15
//               00001      10001  01000     11000  00100          10100  01100        11101
                 KEY_DOWN,  0,     0,        0,     0,             0,     0,           0,    // 16..23
//               00011      10011  01011     11011  00111          10111  01111        11111
                 KEY_END,   0,     0,        0,     KEY_PAGE_DOWN, 0,     0,           0 };  // 24..31

//               00000  10000  01000  11000  00100  10100  01100  11100
byte Keys3[] = { 0,     0,     0,     0,     0,     0,     0,     0,    //  0..7
//               00010  10010  01010  11010  00110  10110  01110  11110
                 0,     0,     0,     0,     0,     0,     0,     0,    //  8..15
//               00001  10001  01000  11000  00100  10100  01100  11101
                 0,     0,     0,     0,     0,     0,     0,     0,    // 16..23
//               00011  10011  01011  11011  00111  10111  01111  11111
                 0,     0,     0,     0,     0,     0,     0,     0 };  // 24..31
                 
byte CurKey = 0;
byte MaxKey = 0;
byte ModKey = 0;
byte KeySet = 0;
byte Send   = 0;

void setup() {
  pinMode(0, INPUT_PULLUP);
  pinMode(1, INPUT_PULLUP);
  pinMode(2, INPUT_PULLUP);
  pinMode(3, INPUT_PULLUP);
  pinMode(4, INPUT_PULLUP);
  pinMode(5, INPUT_PULLUP);
  pinMode(6, INPUT_PULLUP);
}

void loop() {
  button0.update();
  button1.update();
  button2.update();
  button3.update();
  button4.update();
  button5.update();
  button6.update();

// Key Press
  if (button0.fallingEdge()) { MaxKey |=  1; CurKey |=  1; }
  if (button1.fallingEdge()) { MaxKey |=  2; CurKey |=  2; }
  if (button2.fallingEdge()) { MaxKey |=  4; CurKey |=  4; }
  if (button3.fallingEdge()) { MaxKey |=  8; CurKey |=  8; }
  if (button4.fallingEdge()) { MaxKey |= 16; CurKey |= 16; }
  // Mode Keys
  if (button5.fallingEdge()) { KeySet |=  1; KeySet |=  1; }
  if (button6.fallingEdge()) { KeySet |=  2; KeySet |=  2; }
// Key Release
  if (button0.risingEdge()) CurKey &= B11111110;
  if (button1.risingEdge()) CurKey &= B11111101;
  if (button2.risingEdge()) CurKey &= B11111011;
  if (button3.risingEdge()) CurKey &= B11110111;
  if (button4.risingEdge()) CurKey &= B11101111;
  // Mode Keys
  if (button5.risingEdge()) KeySet &= B11111110;
  if (button6.risingEdge()) KeySet &= B11111101;
// Ouput As Needed
  if ((CurKey == 0) && (MaxKey > 0)) {
    Send = 0;
    switch (KeySet) {
      case 0: Send = Keys0[MaxKey]; break;
      case 1: switch (MaxKey) { 
                case  1: if (ModKey == 0) ModKey = 1; else ModKey = 0; break;
                case  3: if (ModKey == 0) ModKey = 2; else ModKey = 0; break;
                default: Send = Keys1[MaxKey]; break;
              }
              break;
      case 2: Send = Keys2[MaxKey]; break;       
    }
    if (Send != 0) {      
      switch (ModKey) {
        case 0: Keyboard.set_modifier(0);
                break;
        case 1: Keyboard.set_modifier(MODIFIERKEY_SHIFT);
                ModKey = 0;
                break;
        case 2: Keyboard.set_modifier(MODIFIERKEY_SHIFT);
                break;
      }
      Keyboard.set_key1(Send);
      Keyboard.send_now();
      Keyboard.set_key1(0);  
      Keyboard.send_now();
    }
    MaxKey = 0;
  }
}

