//#include <Wire.h>
#include <EEPROM.h>

// Five Key Rows
#define 5_00000 0
#define 5_00001 1
#define 5_00010 2
#define 5_00011 3
#define 5_00100 4
#define 5_00101 5
#define 5_00110 6
#define 5_00111 7
#define 5_01000 8
#define 5_01001 9
#define 5_01010 10
#define 5_01011 11
#define 5_01100 12
#define 5_01101 13
#define 5_01110 14
#define 5_01111 15
#define 5_10000 16
#define 5_10001 17
#define 5_10010 18
#define 5_10011 19
#define 5_10100 20
#define 5_10101 21
#define 5_10110 22
#define 5_10111 23
#define 5_11000 24
#define 5_11001 25
#define 5_11010 26
#define 5_11011 27
#define 5_11100 28
#define 5_11101 29
#define 5_11110 30
#define 5_11111 31
// Three Key Rows
#define 3_000  0
#define 3_001  1
#define 3_010  2
#define 3_011  3
#define 3_100  4
#define 3_101  5
#define 3_110  6
#define 3_111  7

// Cursor Keys
// UP, DOWN, LEFT, RIGHT, HOME, END, PAGE UP, PAGE DOWN, TAB (9)393931I

// Normal Keys
// [KEY_A]    .. [KEY_Z]
// [KEY_0]    .. [KEY_9]
// KEY_F1   .. KEY_F12
// KEYPAD_0 .. KEYPAD_9
// KEY_ENTER	    KEY_ESC	    KEY_BACKSPACE   KEY_TAB
// KEY_SPACE	    KEY_MINUS	    KEY_EQUAL	    KEY_LEFT_BRACE
// KEY_RIGHT_BRACE  KEY_BACKSLASH   KEY_NUMBER	    KEY_SEMICOLON
// KEY_QUOTE	    KEY_TILDE       KEY_COMMA	    KEY_PERIOD
// KEY_SLASH	    KEY_CAPS_LOCK   KEY_PRINTSCREEN KEY_SCROLL_LOCK
// KEY_PAUSE	    KEY_INSERT	    KEY_HOME	    KEY_PAGE_UP
// KEY_DELETE	    KEY_END	    KEY_PAGE_DOWN   KEY_RIGHT
// KEY_LEFT	    KEY_DOWN	    KEY_UP	    KEY_NUM_LOCK
// KEYPAD_SLASH	    KEYPAD_ASTERIX  KEYPAD_MINUS    KEYPAD_PLUS
// [KEYPAD_ENTER]	    KEYPAD_PERIOD

byte KeyCombo[] = {
  // *** Single Button Press ***
  B00000000, B00000001,  240,           KEY_ESC,        // [MODE]
                         240,           0,              // Mode 1  
                         240,           0,              // Mode 2
                         240,           0,              // Mode 3  
                         240,           0,              // Mode 4  
  B00000000, B00000010,  KEY_T,         KEY_7,          // [T]
                         KEY_7,         0,              // Mode 1  
                         KEY_PAGE_UP,   0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B00000100,  KEY_O,         KEY_8,          // [O]
                         KEY_8,         0,              // Mode 1  
                         KEY_UP,        0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B00001000,  KEY_I,         KEY_9,          // [I]
                         KEY_9,         0,              // Mode 1  
                         KEY_PAGE_DOWN, 0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B00010000,  KEY_BACKSPACE, KEY_DELETE,     // [Backspace]
                         KEYPAD_MINUS,  KEYPAD_SLASH,   // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B00100000,  255,           254,            // [Shift]   
                         255,           254,            // Mode 1  
                         255,           254,            // Mode 2
                         255,           254,            // Mode 3  
                         255,           254,            // Mode 4  
  B00000000, B01000000,  KEY_A,         KEY_4,          // [A]     
                         KEY_4,         0,              // Mode 1                
                         KEY_LEFT,      0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B10000000,  KEY_E,         KEY_5,          // [E]    
                         KEY_5,         0,              // Mode 1              
                         KEY_DOWN,      0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000001, B00000000,  KEY_N,         KEY_6,          // [N]     
                         KEY_6,         0,              // Mode 1              
                         KEY_RIGHT,     0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000010, B00000000,  KEY_S,         KEYPAD_PLUS,    // [S]           
                         KEYPAD_PLUS,   KEYPAD_ASTERIX, // Mode 1              
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000100, B00000000,  KEY_D,         KEY_1,          // [D]     
                         KEY_1,         0,              // Mode 1              
                         KEY_HOME,      0,              // Mode 2  
                         KEY_F1,        0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00001000, B00000000,  KEY_R,         KEY_2,          // [R]     
                         KEY_2,         0,              // Mode 1              
                         KEY_F2,        0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00010000, B00000000,  KEY_B,         KEY_3,          // [B]     
                         KEY_3,         0,              // Mode 1              
                         KEY_END,       0,              // Mode 2  
                         KEY_F3,        0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00100000, B00000000,  KEY_U,         KEY_0,          // [U]     
                         KEY_0,         0,              // Mode 1              
                         0,             0,              // Mode 2  
                         KEY_F10,       0,              // Mode 3  
                         0,             0,              // Mode 4  
  B01000000, B00000000,  KEY_C,         KEY_PERIOD,     // [C]          
                         KEY_PERIOD,    0,              // Mode 1              
                         0,             0,              // Mode 2  
                         KEY_F11,       0,              // Mode 3  
                         0,             0,              // Mode 4  
  B10000000, B00000000,  KEY_PERIOD,    KEY_EQUAL,      // [.]         
                         KEY_EQUAL,     0,              // Mode 1
                         0,             0,              // Mode 2  
                         KEY_F12,       0,              // Mode 3  
                         0,             0,              // Mode 4  
  // *** Double Button Press ***
  B00000000, B00000011,  241,           KEY_TAB,        // [MODE] + [T]
                         241,           0,              // Mode 1  
                         241,           0,              // Mode 2
                         241,           0,              // Mode 3  
                         241,           0,              // Mode 4  
  B00000000, B00000110,  KEY_F,         KEY_UP,         // [T] + [O]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2      
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B00001100,  KEY_Y,         KEY_DOWN,       // [O] + [I]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B00011000,  KEY_P,         0,              // [I] + [Backspace]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B01100000,  0,             KEY_HOME,       // [Shift] + [A]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B11000000,  KEY_H,         KEY_LEFT,       // [A] + [E]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000001, B10000000,  KEY_M,         KEY_RIGHT,      // [E] + [N]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4 
  B00000011, B00000000,  KEY_G,         KEY_END,        // [N] + [S]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00001100, B00000000,  KEY_L,         0,              // [D] + [R]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00011000, B00000000,  KEY_V,         KEY_END,        // [R] + [B]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00110000, B00000000,  0,             0,              // [U] + [C]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B01100000, B00000000,  KEY_QUOTE,     0,              // [C] + [.]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B11000000, B00000000,  KEY_COMMA,     0,              // [Q] + [X]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  // ** Double Vertical **                         
  B00000000, B00100001,  0,             0,              // [MODE] + [SHIFT]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
                         
  B00000000, B01000010,  KEY_LEFT_BRACE,  0,            // [T] + [A]
                         0,               0,            // Mode 1  
                         0,               0,            // Mode 2  
                         0,               0,            // Mode 3  
                         0,               0,            // Mode 4  
                        
  B00000000, B10000100,  KEY_RIGHT_BRACE, 0,            // [O] + [E]
                         0,               0,            // Mode 1  
                         0,               0,            // Mode 2  
                         0,               0,            // Mode 3  
                         0,               0,            // Mode 4  
  B00000001, B00001000,  KEY_L,           0,            // [I] + [N]
                         0,               0,            // Mode 1  
                         0,               0,            // Mode 2  
                         0,               0,            // Mode 3  
                         0,               0,            // Mode 4  
  B00000001, B00001000,  KEY_L,           0,            // [I] + [N]
                         0,               0,            // Mode 1  
                         0,               0,            // Mode 2  
                         0,               0,            // Mode 3  
                         0,               0,            // Mode 4  
                        
  // *** Split Double Button Press ***
  B00000000, B00000101,  242,           0,              // [MODE] + [I]
                         242,           0,              // Mode 1  
                         242,           0,              // Mode 2  
                         242,           0,              // Mode 3  
                         242,           0,              // Mode 4  
  B00000000, B00001010,  0,             0,              // [T] + [L]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B00010100,  0,             0,              // [I] + [Backspace]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B10100000,  0,             0,              // [Shift] + [A]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000001, B01000000,  0,             0,              // [E] + [S]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000010, B10000000,  0,             0,              // [A] + [.]   
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00010100, B00000000,  0,             0,              // [O] + [C]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B10100000, B00000000,  0,             0,              // [Z] + [X]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  // *** Triple Button Press ***
  B00000000, B00000111,  KEY_Z,         0,              // [MODE] + [T] + [O]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B00001110,  KEY_Q,         KEY_PAGE_UP,    // [T] + [O] + [I]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B00011100,  KEY_J,         0,              // [O] + [I] + [Backspace]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B11100000,  0,             0,              // [Shift] + [A] + [E]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000001, B11000000,  KEY_W,         0,              // [A] + [E] + [N]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3 
                         0,             0,              // Mode 4  
  B00000011, B10000000,  0,             0,              // [E] + [N] + [S]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00011100, B00000000,  KEY_K,         0,              // [D] + [R] + [B]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B11100000, B00000000,  KEY_X,         0,              // [U] + [C] + [.]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  // *** Quadruple Button Press ***
  B00000000, B00001111,  0,             0,              // [MODE] + [T] + [I] + [L]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B00011110,  0,             0,              // [T] + [I] + [L] + [Backspace]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000001, B11100000,  0,             0,              // [Shift] + [E] + [A] + [S]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000011, B11000000,  0,             0,              // [E] + [A] + [S] + [.]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  // *** Quintuple Button Press ***
  B00000000, B00011111,  0,             0,              // [MODE] + [T] + [I] + [L] + [Backspace]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000011, B11100000,  0,             0,              // [Shift] + [E] + [A] + [S] + [.]
                         0,             0,              // Mode 1  
                         0,             0,              // Mode 2  
                         0,             0,              // Mode 3  
                         0,             0,              // Mode 4  
  B00000000, B00001001,  243,           0,              // [MODE] + [L]
                         243,           0,              // Mode 1  
                         243,           0,              // Mode 2
                         243,           0,              // Mode 3  
                         243,           0,              // Mode 4  
  B00000000, B00010001,  244,           0,              // [MODE] + [BS]
                         244,           0,              // Mode 1  
                         244,           0,              // Mode 2
                         244,           0,              // Mode 3  
                         244,           0,              // Mode 4  
  // *** END MARKERS ***
  B11111111, B11111111,  255,           255,            // END MARKER
                         255,           255,            // Mode 1  
                         255,           255,            // Mode 2  
                         255,           255,            // Mode 3  
                         255,           255 };          // Mode 4  

byte OneHand  = true;

byte Send     = 0;
byte SwLo     = 0;
byte SwHi     = 0;
byte SwMod    = 0;
byte LatchLo  = 0;
byte LatchHi  = 0;
byte LatchMod = 0;
byte Sent     = 0;
byte ModKey   = 0;
int  Ptr      = 0;
byte QuitLoop = false;
byte Offset   = 0;
byte Mode     = 0;
byte NewKey   = 0;
int  wLoCnt   = 0;
int  wHiCnt   = 0;
int  wModCnt  = 0;

void setup() {
  //  Inputs - B0 B1 B2 B3 B4 C7 D2 D3 D4 D5 D7 E6 F0 F1 F4 F5 F6 F7 
  // Outputs - B5 B6 B7 C6 D6
  //     I2C - D0 D1
  
/*  // Data Direction Registers - 0: Input, 1: Output
  DDRB = B11100000;
  DDRC = B01000000;
  DDRD = B10000000;
  DDRE = B00000000;
  DDRF = B00000000;    
  // Data Output Registers - Controls Pull-Up On Inputs & Controls Output
  PORTB = B00011111;  // B0 B1 B2 B3 B4
  PORTC = B10111111;  // C7
  PORTD = B01111100;  // D2 D3 D4 D5 D7
  PORTE = B01000000;  // E6
  PORTF = B11110011;  // F0 F1 F4 F5 F6 F7
  //  Wire.begin(); */

  DDRB = B00000000;
  DDRC = B00000000;
  DDRD = B00000000;
  DDRE = B00000000;
  DDRF = B00000000;    
  // Data Output Registers - Controls Pull-Up On Inputs & Controls Output
  PORTB = B11111111;  // B0 B1 B2 B3 B4
  PORTC = B11111111;  // C7
  PORTD = B11111111;  // D2 D3 D4 D5 D7
  PORTE = B11111111;  // E6
  PORTF = B11111111;  // F0 F1 F4 F5 F6 F7
  
  Serial.begin(9600); // USB is always 12 Mbit/sec
}

void loop() {
  // value = EEPROM.read(address);
  // EEPROM.write(address,value);
  
  // F4   F1  F0  B0   B1
  // F5   F6  E6 |D2   B2
  //[F7] [B4] D4  C7  [B3]
  //[  ]  D7  D5 [D3] [  ]
 
  /*
  Serial.print("B:");
  Serial.print(PINB);
  Serial.print(" C:");
  Serial.print(PINC);
  Serial.print(" D:");
  Serial.print(PIND);
  Serial.print(" E:");
  Serial.print(PINE);
  Serial.print(" F:");
  Serial.println(PINF);
 */ 
  // Read Input Pins
  SwLo  =  (PINF & B00010000) ? 1 : 0;  // .0
  SwLo |=  (PINF & B00000010);          // .1
  SwLo |= ((PINF & B00000001) << 2);    // .2
  SwLo |= ((PINB & B00000011) << 3);    // .3 .4
  SwLo |=  (PINF & B01100000);          // .5 .6
  SwLo |= ((PINE & B01000000) << 1);    // .7

  SwHi  =  (PIND & B00000100) ? 1 : 0;  // .0 (D2)
  SwHi |= ((PINB & B00000100) >> 1);    // .1 (B2)
  SwHi |= ((PINB & B00010000) >> 2);    // .2 (B4)
  SwHi |= ((PIND & B00010000) >> 1);    // .3 (D4)
  SwHi |= ((PINC & B10000000) >> 3);    // .4 (C7)
  SwHi |= ((PIND & B01000000) >> 1);    // .5 (D7)
  SwHi |= ((PIND & B00100000) << 1);    // .6 (C7)
  SwHi |= ((PIND & B00001000) << 4);    // .7 (D7)
  
  SwMod  =  (PINF & B10000000) ? 1 : 0;  // .0 (F7)
  SwMod |= ((PINB & B00001000) >> 2);    // .1 (B3)

  SwLo  ^= B11111111;  // Invert Switches
  SwHi  ^= B11111111;  // Invert Switches
  SwMod ^= B00000011;  // Invert Switches

//PORTD = (SwHi || SwLo || SwMod) ? (PORTD | B01000000) : (PORTD & B10111111);
//PORTD |= 128;

  LatchLo  |= SwLo;
  LatchHi  |= SwHi;  
  LatchMod |= SwMod;  

  if (LatchLo != 0) {
     if (SwLo == 0) { if (wLoCnt < 100) wLoCnt += 1; } else wLoCnt = 0; 
  } else wLoCnt = 0;
    
  if (LatchHi != 0) {
     if (SwHi == 0) { if (wHiCnt < 100) wHiCnt += 1; } else wHiCnt = 0; 
  } else wHiCnt = 0;

  if (LatchMod != 0) {
     if (SwMod == 0) { if (wModCnt < 100) wModCnt += 1; } else wModCnt = 0; 
  } else wModCnt = 0;
    
  if ((LatchLo | LatchHi | LatchMod) > 0) delay(1);
  
  if ((LatchLo | LatchHi) > 0){
    if ( ((wLoCnt > 20) || (LatchLo == 0)) && ((wHiCnt > 20) || (LatchHi == 0)) ) {
      Send = 0;
      Ptr  = 0;
      QuitLoop = false;
      while ((Send == 0) && (QuitLoop == false)) {
        if (KeyCombo[Ptr] == LatchHi) {
          if (KeyCombo[(Ptr + 1)] == LatchLo) {
            Offset  = (LatchMod > 0) ? 3 : 2;
            Offset += (Mode * 2);
            Send    = (Offset > 11) ? 0 : KeyCombo[(Ptr + Offset)];
            QuitLoop = true;
          }
        } else if (KeyCombo[Ptr] == B11111111) QuitLoop = true;
        Ptr += 12;
      }
      LatchLo = 0;
      LatchHi = 0;
    }
    if (Send >= 240) {
      if (Send <= 245) Mode = (Send - 240);
      else if (Send >= 250){
        switch (Send) {
          case 255: ModKey = (ModKey > 0)  ? 0 : 1; break;
          case 254: ModKey = (ModKey == 2) ? 0 : 2; break;
        }
      }
      delay(10);
      Sent    = Send;
      Sent    = 0;
    } 
  } 
  
  if ((wModCnt > 20) && LatchMod) {  // Thumb Button Released...
    if (Sent == 0) {
      switch (LatchMod) {
        case 1: Send = KEY_SPACE; break;
        case 2: Send = KEY_ENTER; break;
      }
    }
    LatchMod = 0;
  }

  if (Send > 0) {
 // Serial.print("Send: "); Serial.println(Send);
    if (Send < 240) {
      switch (ModKey) {
        case 0: Keyboard.set_modifier(0); break;
        case 1: Keyboard.set_modifier(MODIFIERKEY_SHIFT); ModKey = 0; break;
        case 2: Keyboard.set_modifier(MODIFIERKEY_SHIFT); break;
      }
      Keyboard.set_key1(Send);
      Keyboard.send_now();
      Keyboard.set_key1(0);  
      Keyboard.send_now();
    }
    delay(20);
    Sent     = Send;
    Send     = 0;
  }
  
  if ( ((wLoCnt > 20) || (LatchLo == 0)) && ((wHiCnt > 20) || (LatchHi == 0)) && ((wModCnt > 20) || (LatchMod == 0)) ) {
    Sent     = 0;
    LatchLo  = 0;
    LatchHi  = 0; 
    LatchMod = 0;
  }
}


