//#include <Wire.h>
//#include <EEPROM.h>

// Five Key Rows  
#define K5_00000 0
#define K5_00001 1
#define K5_00010 2
#define K5_00011 3
#define K5_00100 4
#define K5_00101 5
#define K5_00110 6
#define K5_00111 7
#define K5_01000 8
#define K5_01001 9
#define K5_01010 10
#define K5_01011 11
#define K5_01100 12
#define K5_01101 13
#define K5_01110 14
#define K5_01111 15
#define K5_10000 16
#define K5_10001 17
#define K5_10010 18
#define K5_10011 19
#define K5_10100 20
#define K5_10101 21
#define K5_10110 22
#define K5_10111 23
#define K5_11000 24
#define K5_11001 25
#define K5_11010 26
#define K5_11011 27
#define K5_11100 28
#define K5_11101 29
#define K5_11110 30
#define K5_11111 31

// Three Key Rows
#define K3__000_ 0
#define K3__001_ 1
#define K3__010_ 2
#define K3__011_ 3
#define K3__100_ 4
#define K3__101_ 5
#define K3__110_ 6
#define K3__111_ 7

#define SK_EXCLAMATION (KEY_1 + 0x90)
#define SK_AT          (KEY_2 + 0x90)
#define SK_HASH        (KEY_3 + 0x90)
#define SK_DOLLAR      (KEY_4 + 0x90)
#define SK_PERCENT     (KEY_5 + 0x90)
#define SK_CAROT       (KEY_6 + 0x90)
#define SK_AMP         (KEY_7 + 0x90)
#define SK_STAR        (KEY_8 + 0x90)
#define SK_LBRKT       (KEY_9 + 0x90)
#define SK_RBRKT       (KEY_0 + 0x90)
#define SK_Underline   (KEY_MINUS + 0x90)
#define SK_LCBRKT      (KEY_LEFT_BRACE + 0x90)
#define SK_RCBRKT      (KEY_RIGHT_BRACE + 0x90)
#define SK_QMARK       (KEY_SLASH + 0x90)
#define SK_BAR         (KEY_BACKSLASH + 0x90)
#define SK_LESS        (KEY_COMMA + 0x90) 
#define SK_GRTR        (KEY_PERIOD + 0x90)
#define SK_DQUOTE      (KEY_QUOTE + 0x90)
#define SK_COLON       (KEY_SEMICOLON + 0x90)

/// Cursor Keys
// UP, DOWN, LEFT, RIGHT, HOME, END, PAGE UP, PAGE DOWN, TAB
// Normal Keys
// [KEY_A]  .. [KEY_Z]
// [KEY_0]  .. [KEY_9]
// KEY_F1   .. KEY_F12
// KEYPAD_0 .. KEYPAD_9
// KEY_ENTER	    KEY_ESC	    KEY_BACKSPACE   KEY_TAB
// KEY_SPACE	    KEY_MINUS	    KEY_EQUAL	    KEY_LEFT_BRACE
// KEY_RIGHT_BRACE  KEY_BACKSLASH   KEY_NUMBER	    KEY_SEMICOLON
// KEY_QUOTE	    KEY_TILDE       KEY_COMMA	    KEY_PERIOD
// KEY_SLASH	    KEY_CAPS_LOCK   KEY_PRINTSCREEN KEY_SCROLL_LOCK
// KEY_PAUSE	    KEY_INSERT	    KEY_HOME	    KEY_PAGE_UP
// KEY_DELETE	    KEY_END	    KEY_PAGE_DOWN   KEY_RIGHT
// KEY_LEFT	    KEY_DOWN	    KEY_UP	    KEY_NUM_LOCK
// KEYPAD_SLASH	    KEYPAD_ASTERIX  KEYPAD_MINUS    KEYPAD_PLUS
// KEYPAD_ENTER     KEYPAD_PERIOD

static const byte KeyCombo[] = {
  // *** Single Button Press ***
  K5_10000,   240,           KEY_ESC,        // [MODE]
  K5_00000,   240,           0,              // Mode 1  
  K3__000_,   240,           0,              // Mode 2
  K3__000_,   240,           0,              // Mode 3  
  //--------------------------------------------------
  K5_01000,   KEY_T,         KEY_7,          // [T]
  K5_00000,   KEY_7,         0,              // Mode 1  
  K3__000_,   KEY_PAGE_UP,   0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00100,   KEY_O,         KEY_8,          // [O]
  K5_00000,   KEY_8,         0,              // Mode 1  
  K3__000_,   KEY_UP,        0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00010,   KEY_I,         KEY_9,          // [I]
  K5_00000,   KEY_9,         0,              // Mode 1  
  K3__000_,   KEY_PAGE_DOWN, 0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00001,   KEY_BACKSPACE, KEY_DELETE,     // [Backspace]
  K5_00000,   KEY_BACKSPACE, KEY_DELETE,     // Mode 1  
  K3__000_,   KEY_BACKSPACE, KEY_DELETE,     // Mode 2  
  K3__000_,   KEY_BACKSPACE, KEY_DELETE,     // Mode 3  
  //--------------------------------------------------
  K5_00000,   255,           254,            // [Shift]   
  K5_10000,   255,           254,            // Mode 1  
  K3__000_,   255,           254,            // Mode 2
  K3__000_,   255,           254,            // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_A,         KEY_4,          // [A]     
  K5_01000,   KEY_4,         0,              // Mode 1                
  K3__000_,   KEY_LEFT,      0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_E,         KEY_5,          // [E]    
  K5_00100,   KEY_5,         0,              // Mode 1              
  K3__000_,   KEY_DOWN,      0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_N,         KEY_6,          // [N]     
  K5_00010,   KEY_6,         0,              // Mode 1              
  K3__000_,   KEY_RIGHT,     0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_S,         KEYPAD_PLUS,    // [S]           
  K5_00001,   KEYPAD_PLUS,   KEYPAD_ASTERIX, // Mode 1              
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_D,         KEY_1,          // [D]     
  K5_00000,   KEY_1,         0,              // Mode 1              
  K3__100_,   KEY_HOME,      0,              // Mode 2  
  K3__000_,   KEY_F1,        0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_R,         KEY_2,          // [R]     
  K5_00000,   KEY_2,         0,              // Mode 1              
  K3__010_,   KEY_F2,        0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_B,         KEY_3,          // [B]     
  K5_00000,   KEY_3,         0,              // Mode 1           
  K3__001_,   KEY_END,       0,              // Mode 2  
  K3__000_,   KEY_F3,        0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_U,         KEY_0,          // [U]     
  K5_00000,   KEY_0,         0,              // Mode 1              
  K3__000_,   0,             0,              // Mode 2  
  K3__100_,   KEY_F10,       0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_C,         KEY_PERIOD,     // [C]          
  K5_00000,   KEY_PERIOD,    0,              // Mode 1              
  K3__000_,   0,             0,              // Mode 2  
  K3__010_,   KEY_F11,       0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_PERIOD,    KEY_EQUAL,      // [.]         
  K5_00000,   KEY_EQUAL,     0,              // Mode 1
  K3__000_,   0,             0,              // Mode 2  
  K3__001_,   KEY_F12,       0,              // Mode 3  
  //--------------------------------------------------
  // *** Double Button Press ***
  K5_11000,   241,           KEY_TAB,        // [MODE] + [T]
  K5_00000,   241,           0,              // Mode 1  
  K3__000_,   241,           0,              // Mode 2
  K3__000_,   241,           0,              // Mode 3  
  //--------------------------------------------------
  K5_01100,   KEY_F,         KEY_UP,         // [T] + [O]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2      
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00110,   KEY_Y,         KEY_DOWN,       // [O] + [I]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3
  //--------------------------------------------------
  K5_00011,   KEY_Q,         0,              // [I] + [Backspace]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             KEY_HOME,       // [Shift] + [A]
  K5_11000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_H,         KEY_LEFT,       // [A] + [E]
  K5_01100,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_M,         KEY_RIGHT,      // [E] + [N]
  K5_00110,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_G,         KEY_END,        // [N] + [S]
  K5_00011,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_L,         KEYPAD_MINUS,   // [D] + [R]
  K5_00000,   0,             0,              // Mode 1  
  K3__110_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_V,         KEYPAD_PLUS,    // [R] + [B]
  K5_00000,   0,             0,              // Mode 1  
  K3__011_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_QUOTE,     253,            // [U] + [C]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__110_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_COMMA,     252,              // [C] + [.]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__011_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // ** Diagonal Doubles **
  K5_00100,   KEY_SLASH,     SK_LESS,        // [O] + [A]
  K5_01000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00100,   KEY_BACKSLASH, SK_GRTR,        // [O] + [N]
  K5_00010,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_L,         SK_LCBRKT,      // [O] + [A]
  K5_00100,   0,             0,              // Mode 1  
  K3__100_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             SK_RCBRKT,      // [O] + [A]
  K5_00100,   0,             0,              // Mode 1  
  K3__001_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
 //--------------------------------------------------
  K5_00000,   SK_LESS,       KEY_SLASH,      // [O] + [A]
  K5_10000,   0,             0,              // Mode 1  
  K3__100_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_GRTR,       KEYPAD_ASTERIX, // [O] + [A]
  K5_00001,   0,             0,              // Mode 1  
  K3__001_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // ** Double Vertical **                         
  K5_10000,   KEY_LEFT_BRACE,0,              // [T] + [A]
  K5_10000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_01000,   SK_LBRKT,      0,              // [MODE] + [SHIFT]
  K5_01000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00100,   SK_EXCLAMATION, 0,             // [O] + [E]
  K5_00100,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00010,   SK_RBRKT,      0,              // [BACKSPACE] + [S]
  K5_00010,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00001,   KEY_RIGHT_BRACE, 0,              // [I] + [N]
  K5_00001,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_LCBRKT,     0,              // [BACKSPACE] + [S]
  K5_01000,   0,             0,              // Mode 1  lv
  K3__100_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_BAR,        0,             // [BACKSPACE] + [S]
  K5_00100,   0,             0,              // Mode 1  
  K3__010_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_RCBRKT,     0,              // [BACKSPACE] + [S]
  K5_00010,   0,             0,              // Mode 1  
  K3__001_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,             // [BACKSPACE] + [S]
  K5_00000,   0,             0,              // Mode 1  
  K3__100_,   0,             0,              // Mode 2  
  K3__100_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_SEMICOLON, 0,              // [BACKSPACE] + [S]
  K5_00000,   0,             0,              // Mode 1  
  K3__010_,   0,             0,              // Mode 2  
  K3__010_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_COLON,      0,              // [BACKSPACE] + [S]
  K5_00000,   0,             0,              // Mode 1  
  K3__001_,   0,             0,              // Mode 2  
  K3__001_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // *** Split Double Button Press ***
  K5_10100,   242,           0,              // [MODE] + [I]
  K5_00000,   242,           0,              // Mode 1  
  K3__000_,   242,           0,              // Mode 2  
  K3__000_,   242,           0,              // Mode 3  
  //--------------------------------------------------
  K5_01010,   KEY_U,         0,              // [T] + [L]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00101,   0,             0,              // [I] + [Backspace]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,              // [Shift] + [A]
  K5_10100,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_W,         0,              // [E] + [S]
  K5_01010,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,              // [A] + [.]   
  K5_00101,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_C,         SK_STAR,        // [O] + [C]
  K5_00000,   0,             0,              // Mode 1  
  K3__101_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_DQUOTE,     0,              // [Z] + [X]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__101_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // *** Triple Button Press ***
  K5_11100,   KEY_Z,         0,              // [MODE] + [T] + [O]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_01110,   KEY_P,         KEY_PAGE_UP,    // [T] + [O] + [I]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00111,   KEY_J,         0,              // [O] + [I] + [Backspace]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,              // [Shift] + [A] + [E]
  K5_11100,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_W,         0,              // [A] + [E] + [N]
  K5_01110,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3 
  //--------------------------------------------------
  K5_00000,   SK_QMARK,      0,              // [E] + [N] + [S]
  K5_00111,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_K,         0,              // [D] + [R] + [B]
  K5_00000,   0,             0,              // Mode 1  
  K3__111_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_X,         0,              // [U] + [C] + [.]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__111_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00100,   SK_CAROT,      0,              // [U] + [C] + [.]
  K5_01010,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // *** Quadruple Button Press ***
  K5_11110,   0,             0,              // [MODE] + [T] + [I] + [L]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_01111,   0,             0,              // [T] + [I] + [L] + [Backspace]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,              // [Shift] + [E] + [A] + [S]
  K5_11110,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,              // [E] + [A] + [S] + [.]
  K5_01111,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // *** Quintuple Button Press ***
  K5_11111,   0,             0,              // [MODE] + [T] + [I] + [L] + [Backspace]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,              // [Shift] + [E] + [A] + [S] + [.]
  K5_11111,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K3__000_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_10010,   243,           0,              // [MODE] + [L]
  K5_00000,   243,           0,              // Mode 1  
  K3__000_,   243,           0,              // Mode 2
  K3__000_,   243,           0,              // Mode 3  
  //--------------------------------------------------
  K5_10001,   244,           0,              // [MODE] + [BS]
  K5_00000,   244,           0,              // Mode 1  
  K3__000_,   244,           0,              // Mode 2
  K3__000_,   244,           0,              // Mode 3  
  //--------------------------------------------------
  // END MARKER --------------------------------------
  255 };                                     // END MARKER
  //===================================================

byte Row[]    = { 0, 0, 0, 0, 0 };  // First Element Is Modifier
byte oRow[]   = { 1, 1, 1, 1, 1 };  // First Element Is Modifier
byte Latch[]  = { 0, 0, 0, 0, 0 };  // Latched Keys
byte Count[]  = { 0, 0, 0, 0, 0 };  // Key Up Count
byte RowsUp   = 0;                  // Key Rows Inactive

byte i;
byte Tbit;
byte Keys;
byte Key;
byte Send     = 0;
byte Send2    = 0;
byte Sent     = 0;

byte ModKey   = 0;
int  Ptr      = 0;
byte QuitLoop = false;
byte Offset   = 0;
byte Mode     = 0;
byte NewKey   = 0;
byte Latched  = 0;
byte Mirror   = 0;
byte Modifier = 0;

byte Repeated = 0;
long Now;
long bounce[5];

void setup() {
  // Data Direction Registers - 0: Input, 1: Output
  DDRB  = B11100000;
  DDRC  = B01000000;
  DDRD  = B10000000;
  DDRE  = B00000000;
  DDRF  = B00000000;    
  // Data Output Registers - Controls Pull-Up On Inputs & Controls Output
  PORTB = B00011111;  // B0 B1 B2 B3 B4 B6
  PORTC = B10111111;  // C7
  PORTD = B01111111;  // D2 D3 D4 D5 D7
  PORTE = B11111111;  // E6
  PORTF = B11111111;  // F0 F1 F4 F5 F6 F7
// Serial.begin(9600); // USB is always 12 Mbit/sec
}

void loop() {
  // value = EEPROM.read(address);
  // EEPROM.write(address,value);
  
  //(B5)(B6)(D7)(B7)(C6)
  // F4  F1  F0  B0  B1
  // F5  F6  E6  D2  B2
  //[F7] B4  D4  C7 [B3]
  //[  ] D6  D5  D3 [  ]

  if (Mirror == 0) { // Right Hand Mode
    // Write LEDs
    if (Mode == 0) PORTB |= B00100000; else PORTB &= B11011111;
    if (Mode == 1) PORTB |= B01000000; else PORTB &= B10111111;
    if (Mode == 2) PORTD |= B10000000; else PORTD &= B01111111;
    if (Mode == 3) PORTB |= B10000000; else PORTB &= B01111111;
    if (Mode == 4) PORTC |= B01000000; else PORTC &= B10111111;
    // Read Input Pins
    Keys   =  (PINF & B10000000) ? 1 : 0;  // .0 
    Keys  |= ((PINB & B00001000) >> 2);    // .1 
    Row[0] =   Keys ^ B00000011;  // Invert Switches
    Keys   =  (PINF & B00010000);          // .4
    Keys  |= ((PINF & B00000011) << 2);    // .3 .2
    Keys  |= ((PINB & B00000001) << 1);    // .1
    Keys  |= ((PINB & B00000010) >> 1);    // .0
    Row[1] =   Keys ^ B00011111;
    Keys   = ((PINF & B00100000) >> 1);    // .4
    Keys  |= ((PINF & B01000000) >> 3);    // .3
    Keys  |= ((PINE & B01000000) >> 4);    // .2
    Keys  |= ((PIND & B00000100) >> 1);    // .1
    Keys  |= ((PINB & B00000100) >> 2);    // .0
    Row[2] =   Keys ^ B00011111;
    Keys   = ((PINB & B00010000) >> 2);    // .2
    Keys  |= ((PIND & B00010000) >> 3);    // .1
    Keys  |= ((PINC & B10000000) >> 7);    // .0
    Row[3] =   Keys ^ B00000111;
    Keys   = ((PIND & B01100000) >> 4);    // .2 .1
    Keys  |= ((PIND & B00001000) >> 3);    // .0
    Row[4] =   Keys ^ B00000111;
  } else {   // Left Hand Mode
    // Write LEDs
    if (Mode == 0) PORTC |= B01000000; else PORTC &= B10111111;    
    if (Mode == 1) PORTB |= B10000000; else PORTB &= B01111111;
    if (Mode == 2) PORTD |= B10000000; else PORTD &= B01111111;
    if (Mode == 3) PORTB |= B01000000; else PORTB &= B10111111;
    if (Mode == 4) PORTB |= B00100000; else PORTB &= B11011111;
    // Read Input Pins
    Keys   =  (PINF & B10000000) ? 2 : 0;  // .0 
    Keys  |= ((PINB & B00001000) >> 3);    // .1 
    Row[0] =   Keys ^ B00000011;  // Invert Switches    
    Keys   = ((PINF & B00010000) >> 4);    // .0
    Keys  |=  (PINF & B00000010);          // .1
    Keys  |= ((PINF & B00000001) << 2);    // .2
    Keys  |= ((PINB & B00000011) << 3);    // .3 .4
    Row[1] =   Keys ^ B00011111;
    Keys   = ((PINF & B01100000) >> 5);    // .0 .1
    Keys  |= ((PINE & B01000000) >> 4);    // .2
    Keys  |= ((PIND & B00000100) << 1);    // .3
    Keys  |= ((PINB & B00000100) << 2);    // .4
    Row[2] =   Keys ^ B00011111;
    Keys   = ((PINB & B00010000) >> 4);    // .0
    Keys  |= ((PIND & B00010000) >> 3);    // .1
    Keys  |= ((PINC & B10000000) >> 5);    // .2
    Row[3] =   Keys ^ B00000111;
    Keys   = ((PIND & B01000000) >> 6);    // .0
    Keys  |= ((PIND & B00100000) >> 4);    // .1
    Keys  |= ((PIND & B00001000) >> 1);    // .2
    Row[4] =   Keys ^ B00000111;
  }
  Keys    = 0;
  Latched = 0;
  Tbit    = 1;
  RowsUp  = 0;
  Now = millis();
  for (i=0; i<5; i++) {
    Key       = Row[i];
    if (Key != oRow[i]) bounce[i] = Now;
    oRow[i]   = Key;    
    Keys     |= Key;
    Latch[i] |= Key;
    if (i > 0) Latched |= Latch[i];
    if (((Key == 0) && ((Now - bounce[i]) > 10)) || (Latch[i] == 0)) RowsUp |= Tbit;
    if ((i > 0) && (Key != 0) && ((Now - bounce[i]) > 750)) {
      RowsUp   |= Tbit;
      bounce[i] = (Now - 600);
      Repeated  = true;
    }
    Tbit <<= 1;
  }
  if (Latched > 0) {
    if ((RowsUp > 29) && ((Keys != 0) || !Repeated))  {
      Repeated = false;
      Send = 0;
      Ptr  = 0;
      QuitLoop = false;
      while ((Send == 0) && (QuitLoop == false)) {
        if (KeyCombo[Ptr] == Latch[1]) {
          if (KeyCombo[(Ptr + 3)] == Latch[2]) {
            if (KeyCombo[(Ptr + 6)] == Latch[3]) {
              if (KeyCombo[(Ptr + 9)] == Latch[4]) {
                Offset   = (Latch[0] > 0) ? 2 : 1;
                if ((Mode > 0) && (Mode < 4)) Offset  += (Mode * 3);
                Send = (Offset > 11) ? 0 : KeyCombo[(Ptr + Offset)];
                QuitLoop = true;
              }
            }
          }
        } else if (KeyCombo[Ptr] == 255) QuitLoop = true;
        Ptr += 12;
      }
      Latch[1] = 0;
      Latch[2] = 0;
      Latch[3] = 0;
      Latch[4] = 0;
    }
    if ((Send > 0xA5) && (Send < 0xDF)) {
      ModKey = 1;
      Send  -= 0x90;
    } else {
      if (Send >= 240) {
        if (Send < 244) Mode = (Send - 240);
        else {
          if (Send == 244) {
            Mode   = 0;
            Mirror = (1 - Mirror);
          } else if (Send >= 250){
            switch (Send) {
              case 255: ModKey = ((ModKey & 3) == 0) ? 1 : 0; break;  // Shift
              case 254: ModKey = ((ModKey & 3) < 2) ? 2 : 0; break;  // Shift Lock
              case 253: ModKey = (ModKey ^ 4); break;  // Control
              case 252: ModKey = (ModKey ^ 8); break;  // Alt
            }
          }
        }
        delay(10);
        Sent = Send;
        Sent = 0;
      }
    }
  } 
  if (RowsUp & 1) {  // Thumb Button Released...
    if (Sent == 0) {
      switch (Latch[0]) {
        case 1: Send = KEY_SPACE; break;
        case 2: Send = KEY_ENTER; break;
      }
    }
    Sent = 0;
    Latch[0] = 0;
  }
  if (Send > 0) {
    if (Send < 240) {      
      Send2 = 0;
      if ((Send == (char)KEY_9) && (ModKey > 0)) Send2 = KEY_0;
      if (Send == (char)KEY_LEFT_BRACE) Send2 = KEY_RIGHT_BRACE;
      Modifier = 0;
      if ((ModKey & 3) > 0) Modifier  = MODIFIERKEY_SHIFT;
      if ((ModKey & 4) > 0) Modifier |= MODIFIERKEY_CTRL;
      if ((ModKey & 8) > 0) Modifier |= MODIFIERKEY_ALT;
      Keyboard.set_modifier(Modifier);
      Keyboard.set_key1(Send);
      Keyboard.set_key2(Send2);
      Keyboard.send_now();
      if (Send2 != 0) {
        // Send the cursor left with no modifier
        Keyboard.set_modifier(0);
        Keyboard.set_key1(KEY_LEFT);
        Keyboard.set_key2(0);
        Keyboard.send_now();
      }
      Keyboard.set_modifier(0);
      Keyboard.set_key1(0);
      Keyboard.send_now();
       ModKey &= B11110010;
    }
    Sent  = Send;
    Send  = 0;
    Send2 = 0;
  }
}



