// OneHand V0.4
//#define OLED

#ifdef OLED
  #include <Wire.h>
  #include <SeeedOLED.h>
#endif
//#include <EEPROM.h>

// Five Key Rows  
#define K5_00000 0
#define K5_00001 1
#define K5_00010 2
#define K5_00011 3
#define K5_00100 4
#define K5_00101 5
#define K5_00110 6
#define K5_00111 7
#define K5_01000 8
#define K5_01001 9
#define K5_01010 10
#define K5_01011 11
#define K5_01100 12
#define K5_01101 13
#define K5_01110 14
#define K5_01111 15
#define K5_10000 16
#define K5_10001 17
#define K5_10010 18
#define K5_10011 19
#define K5_10100 20
#define K5_10101 21
#define K5_10110 22
#define K5_10111 23
#define K5_11000 24
#define K5_11001 25
#define K5_11010 26
#define K5_11011 27
#define K5_11100 28
#define K5_11101 29
#define K5_11110 30
#define K5_11111 31

// Three Key Row
#define K3__000_ 0
#define K3__001_ 1
#define K3__010_ 2
#define K3__011_ 3
#define K3__100_ 4
#define K3__101_ 5
#define K3__110_ 6
#define K3__111_ 7

// Two Key Row
#define K2___00_ 0
#define K2___01_ 1
#define K2___10_ 2
#define K2___11_ 3

#define SK_EXCLAMATION (KEY_1 + 0x90)
#define SK_AT          (KEY_2 + 0x90)
#define SK_HASH        (KEY_3 + 0x90)
#define SK_DOLLAR      (KEY_4 + 0x90)
#define SK_PERCENT     (KEY_5 + 0x90)
#define SK_CAROT       (KEY_6 + 0x90)
#define SK_AMP         (KEY_7 + 0x90)
#define SK_STAR        (KEY_8 + 0x90)
#define SK_LBRKT       (KEY_9 + 0x90)
#define SK_RBRKT       (KEY_0 + 0x90)
#define SK_Underline   (KEY_MINUS + 0x90)
#define SK_LCBRKT      (KEY_LEFT_BRACE + 0x90)
#define SK_QMARK       (KEY_SLASH + 0x90)
#define SK_BAR         (KEY_BACKSLASH + 0x90)
#define SK_LESS        (KEY_COMMA + 0x90) 
#define SK_GRTR        (KEY_PERIOD + 0x90)
#define SK_DQUOTE      (KEY_QUOTE + 0x90)
#define SK_COLON       (KEY_SEMICOLON + 0x90)

/// Cursor Keys
// UP, DOWN, LEFT, RIGHT, HOME, END, PAGE UP, PAGE DOWN, TAB
// Normal Keys
// [KEY_A]  .. [KEY_Z]
// [KEY_0]  .. [KEY_9]
// KEY_F1   .. KEY_F12
// KEYPAD_0 .. KEYPAD_9
// KEY_ENTER	    KEY_ESC	    KEY_BACKSPACE   KEY_TAB
// KEY_SPACE	    KEY_MINUS	    KEY_EQUAL	    KEY_LEFT_BRACE
// KEY_RIGHT_BRACE  KEY_BACKSLASH   KEY_NUMBER	    KEY_SEMICOLON
// KEY_QUOTE	    KEY_TILDE       KEY_COMMA	    KEY_PERIOD
// KEY_SLASH	    KEY_CAPS_LOCK   KEY_PRINTSCREEN KEY_SCROLL_LOCK
// KEY_PAUSE	    KEY_INSERT	    KEY_HOME	    KEY_PAGE_UP
// KEY_DELETE	    KEY_END	    KEY_PAGE_DOWN   KEY_RIGHT
// KEY_LEFT	    KEY_DOWN	    KEY_UP	    KEY_NUM_LOCK
// KEYPAD_SLASH	    KEYPAD_ASTERIX  KEYPAD_MINUS    KEYPAD_PLUS
// KEYPAD_ENTER     KEYPAD_PERIOD

static const char KeyCombo[] = {
  // *** Single Button Press ***
  K5_10000,   240,           KEY_ESC,        // [MODE]
  K5_00000,   240,           0,              // Mode 1  
  K3__000_,   240,           0,              // Mode 2
  K2___00_,   240,           0,              // Mode 3  
  //--------------------------------------------------
  K5_01000,   KEY_A,         KEY_7,          // [T]
  K5_00000,   KEY_7,         0,              // Mode 1  
  K3__000_,   KEY_PAGE_UP,   0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00100,   KEY_O,         KEY_8,          // [O]
  K5_00000,   KEY_8,         0,              // Mode 1  
  K3__000_,   KEY_UP,        0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00010,   KEY_I,         KEY_9,          // [I]
  K5_00000,   KEY_9,         0,              // Mode 1  
  K3__000_,   KEY_PAGE_DOWN, 0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00001,   KEY_BACKSPACE, KEY_DELETE,     // [Backspace]
  K5_00000,   KEY_BACKSPACE, KEY_DELETE,     // Mode 1  
  K3__000_,   KEY_BACKSPACE, KEY_DELETE,     // Mode 2  
  K2___00_,   KEY_BACKSPACE, KEY_DELETE,     // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_H,         KEY_0,          // [Shift]   
  K5_10000,   KEY_0,         0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_T,         KEY_4,          // [A]     
  K5_01000,   KEY_4,         0,              // Mode 1                
  K3__000_,   KEY_LEFT,      0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_E,         KEY_5,          // [E]    
  K5_00100,   KEY_5,         0,              // Mode 1              
  K3__000_,   KEY_DOWN,      0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_N,         KEY_6,          // [N]     
  K5_00010,   KEY_6,         0,              // Mode 1              
  K3__000_,   KEY_RIGHT,     0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_S,         KEYPAD_PLUS,    // [S]           
  K5_00001,   KEYPAD_PLUS,   KEYPAD_ASTERIX, // Mode 1              
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_D,         KEY_1,          // [D]     
  K5_00000,   KEY_1,         0,              // Mode 1              
  K3__100_,   KEY_HOME,      0,              // Mode 2  
  K2___00_,   KEY_F1,        0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_R,         KEY_2,          // [R]     
  K5_00000,   KEY_2,         0,              // Mode 1              
  K3__010_,   KEY_F2,        0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_L,         KEY_3,          // [B]     
  K5_00000,   KEY_3,         0,              // Mode 1           
  K3__001_,   KEY_END,       0,              // Mode 2  
  K2___00_,   KEY_F3,        0,              // Mode 3  
  //--------------------------------------------------
//  K5_00000,   255,           254,            // [U]     
//  K5_00000,   255,           255,            // Mode 1              
//  K3__000_,   255,           255,            // Mode 2  
//  K3__100_,   KEY_F10,       0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_C,         KEY_PERIOD,     // [C]          
  K5_00000,   KEY_PERIOD,    0,              // Mode 1              
  K3__000_,   0,             0,              // Mode 2  
  K2___10_,   KEY_F11,       0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_PERIOD,    KEY_EQUAL,      // [.]         
  K5_00000,   KEY_EQUAL,     0,              // Mode 1
  K3__000_,   0,             0,              // Mode 2  
  K2___01_,   KEY_F12,       0,              // Mode 3  
  //--------------------------------------------------
  // *** Double Button Press ***
  K5_11000,   241,           KEY_TAB,        // [MODE] + [T]
  K5_00000,   241,           0,              // Mode 1  
  K3__000_,   241,           0,              // Mode 2
  K2___00_,   241,           0,              // Mode 3  
  //--------------------------------------------------
  K5_01100,   KEY_F,         KEY_UP,         // [T] + [O]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2      
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00110,   KEY_Y,         KEY_DOWN,       // [O] + [I]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3
  //--------------------------------------------------
  K5_00011,   KEY_Q,         0,              // [I] + [Backspace]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_C,         KEY_HOME,       // [Shift] + [A]
  K5_11000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_U,         KEY_LEFT,       // [A] + [E]
  K5_01100,   KEYPAD_MINUS,  0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_M,         KEY_RIGHT,      // [E] + [N]
  K5_00110,   KEYPAD_PLUS,   0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_G,         KEY_END,        // [N] + [S]
  K5_00011,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_B,         KEYPAD_MINUS,   // [D] + [R]
  K5_00000,   KEY_SLASH,     0,              // Mode 1  
  K3__110_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_V,         KEYPAD_PLUS,    // [R] + [B]
  K5_00000,   SK_STAR,       0,              // Mode 1  
  K3__011_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
//  K5_00000,   KEY_QUOTE,     253,            // [U] + [C]
//  K5_00000,   0,             0,              // Mode 1  
//  K3__000_,   0,             0,              // Mode 2  
//  K3__110_,   0,             0,              // Mode 3  $
  //--------------------------------------------------
  K5_00000,   KEY_COMMA,     252,              // [C] + [.]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___11_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // ** Diagonal Doubles **
  K5_00100,   KEY_SLASH,     SK_LESS,        // [O] + [A]
  K5_01000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00100,   KEY_BACKSLASH, SK_GRTR,        // [O] + [N]
  K5_00010,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,              // [O] + [A]
  K5_00100,   0,             0,              // Mode 1  
  K3__100_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,              // [O] + [A]
  K5_00100,   0,             0,              // Mode 1  
  K3__001_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
 //--------------------------------------------------
  K5_00000,   SK_LESS,       KEY_SLASH,      // [O] + [A]
  K5_10000,   0,             0,              // Mode 1  
  K3__100_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_GRTR,       KEYPAD_ASTERIX, // [O] + [A]
  K5_00001,   0,             0,              // Mode 1  
  K3__001_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // ** Double Vertical **                         
  K5_10000,   KEY_LEFT_BRACE,0,              // [T] + [A]
  K5_10000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_01000,   SK_LBRKT,      0,              // [MODE] + [SHIFT]
  K5_01000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00100,   SK_EXCLAMATION, 0,             // [O] + [E]
  K5_00100,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00010,   SK_RBRKT,      0,              // [BACKSPACE] + [S]
  K5_00010,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00001,   KEY_RIGHT_BRACE, 0,              // [I] + [N]
  K5_00001,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_LCBRKT,     0,              // [BACKSPACE] + [S]
  K5_01000,   0,             0,              // Mode 1
  K3__100_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_BAR,        0,             // [BACKSPACE] + [S]
  K5_00100,   0,             0,              // Mode 1  
  K3__010_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_RBRKT,     0,              // [BACKSPACE] + [S]
  K5_00010,   0,             0,              // Mode 1  
  K3__001_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_AT,         0,             // [BACKSPACE] + [S]
  K5_00000,   0,             0,              // Mode 1  
  K3__100_,   0,             0,              // Mode 2  
  K2___10_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_SEMICOLON, 0,              // [BACKSPACE] + [S]
  K5_00000,   0,             0,              // Mode 1  
  K3__010_,   0,             0,              // Mode 2  
  K2___10_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   SK_COLON,      0,              // [BACKSPACE] + [S]
  K5_00000,   0,             0,              // Mode 1  
  K3__001_,   0,             0,              // Mode 2  
  K2___01_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // *** Split Double Button Press ***
  K5_10100,   242,           0,              // [MODE] + [I]
  K5_00000,   242,           0,              // Mode 1  
  K3__000_,   242,           0,              // Mode 2  
  K2___00_,   242,           0,              // Mode 3  
  //--------------------------------------------------
  K5_01010,   KEY_P,         0,              // [T] + [L]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00101,   0,             0,              // [I] + [Backspace]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,              // [Shift] + [A]
  K5_10100,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_W,         0,              // [E] + [S]
  K5_01010,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,              // [A] + [.]   
  K5_00101,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_C,         SK_STAR,        // [O] + [C]
  K5_00000,   0,             0,              // Mode 1  
  K3__101_,   0,             0,              // Mode 2 
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
//  K5_00000,   SK_DQUOTE,     251,            // [Z] + [X]
//  K5_00000,   0,             0,              // Mode 1  
//  K3__000_,   0,             0,              // Mode 2  
//  K3__101_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // *** Triple Button Press ***
  K5_11100,   KEY_Z,         0,              // [MODE] + [T] + [O]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_01110,   KEY_J,         KEY_PAGE_UP,    // [T] + [O] + [I]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00111,   0,             0,              // [O] + [I] + [Backspace]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_X,         0,              // [Shift] + [A] + [E]
  K5_11100,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_W,         0,              // [A] + [E] + [N]
  K5_01110,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3 
  //--------------------------------------------------
  K5_00000,   SK_QMARK,      0,              // [E] + [N] + [S]
  K5_00111,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_K,         0,              // [D] + [R] + [B]
  K5_00000,   0,             0,              // Mode 1  
  K3__111_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
//  K5_00000,   KEY_X,         0,              // [U] + [C] + [.]
//  K5_00000,   0,             0,              // Mode 1  
//  K3__000_,   0,             0,              // Mode 2  
//  K3__111_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00100,   SK_CAROT,      0,              // [U] + [C] + [.]
  K5_01010,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // *** Quadruple Button Press ***
  K5_11110,   0,             0,              // [MODE] + [T] + [I] + [L]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_01111,   0,             0,              // [T] + [I] + [L] + [Backspace]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   0,             0,              // [Shift] + [E] + [A] + [S]
  K5_11110,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_BACKSPACE, 0,              // [E] + [A] + [S] + [.]
  K5_01111,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  // *** Quintuple Button Press ***
  K5_11111,   0,             0,              // [MODE] + [T] + [I] + [L] + [Backspace]
  K5_00000,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_00000,   KEY_ESC,       0,              // [Shift] + [E] + [A] + [S] + [.]
  K5_11111,   0,             0,              // Mode 1  
  K3__000_,   0,             0,              // Mode 2  
  K2___00_,   0,             0,              // Mode 3  
  //--------------------------------------------------
  K5_10010,   243,           0,              // [MODE] + [L]
  K5_00000,   243,           0,              // Mode 1  
  K3__000_,   243,           0,              // Mode 2
  K2___00_,   243,           0,              // Mode 3  
  //--------------------------------------------------
  K5_10001,   244,           0,              // [MODE] + [BS]
  K5_00000,   244,           0,              // Mode 1  
  K3__000_,   244,           0,              // Mode 2
  K2___00_,   244,           0,              // Mode 3  
  //--------------------------------------------------
  // END MARKER --------------------------------------
  255 };                                     // END MARKER
  //===================================================

byte  Row[]    = { 0, 0, 0, 0, 0 };  // First Element Is Modifier
//byte  oRow[]   = { 1, 1, 1, 1, 1 };  // First Element Is Modifier
//byte  Latch[]  = { 0, 0, 0, 0, 0 };  // Latched Keys
//byte  First[]  = { 0, 0, 0, 0, 0 };  // First Latched Key
//byte  Seq[]    = { 0, 0, 0, 0, 0 };  // Combination pressed as sequence
//byte  Count[]  = { 0, 0, 0, 0, 0 };  // Key Up Count
//byte  RowsUp   = 0;                  // Key Rows Inactive

byte  i;
byte  Keys;
byte  Send[4]  = {0, 0, 0, 0};
byte  Mod[4]   = {0, 0, 0, 0};
byte  SendOld  = 0;

byte  nThumb   = 0;
byte  Thumb    = 0;
byte  oThumb   = 0;
long  lThumb   = 0;

byte  ModKey   = 0;
byte  oMod     = 0;
int   Ptr      = 0;
byte  QuitLoop = false;
byte  Offset   = 0;
byte  Mode     = 0;
byte  oMode    = 0;
byte  OldMode  = 0;
byte  Mirror   = 0;
byte  Modifier = 0;
byte  Step = 0;
byte  Repeated = false;

unsigned int iCombo = 0;
unsigned int iComboOld = 0;
unsigned int iLatch = 0;
unsigned int iLatchOld = 0;
unsigned int iSeq[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
unsigned int iSendCode = 0;
unsigned int iCode = 0;

long  Now;
long  lBounce;
long  keydelay;

void setup() {
  // Data Direction Registers - 0: Input, 1: Output  
  DDRB  = B11100000;
  DDRC  = B01000000;
  DDRD  = B10000000;
  DDRE  = B00000000;
  DDRF  = B00000000;    
  // Data Output Registers - Controls Pull-Up On Inputs & Controls Output
  PORTB = B00011111;  // B0 B1 B2 B3 B4 B6
  PORTC = B10111111;  // C7
  PORTD = B01111111;  // D2 D3 D4 D5 D7
  PORTE = B11111111;  // E6
  PORTF = B11111111;  // F0 F1 F4 F5 F6 F7
  Serial.begin(9600); // USB is always 12 Mbit/sec
//  FlashNow = millis();

//  Serial.println("Hello");

#ifdef OLED
  Wire.begin();
  SeeedOled.init();                    // initialze SEEED OLED display
  SeeedOled.clearDisplay();            // clear the screen and set start position to top left corner
  SeeedOled.setNormalDisplay();        // Set display to normal mode (i.e non-inverse mode)
  SeeedOled.setPageMode();             // Set addressing mode to Page Mode
  SeeedOled.setTextXY(0,0); SeeedOled.putString(" Standard Mode"); // Print the String
  SeeedOled.setTextXY(2,0); SeeedOled.putString(" SHIFT - OFF"); // Print the String
  SeeedOled.setTextXY(3,0); SeeedOled.putString("  CTRL - OFF"); // Print the String
  SeeedOled.setTextXY(4,0); SeeedOled.putString("   ALT - OFF"); // Print the String
  SeeedOled.setTextXY(5,0); SeeedOled.putString("   WIN - OFF"); // Print the String
#endif
}

void loop() {
  // value = EEPROM.read(address);
  // EEPROM.write(address,value);
  
  //(B5)(B6)(D7)(B7)(C6)
  // F4  F1  F0  B0  B1
  // F5  F6  E6  D2  B2
  //[F7] B4  D4  C7 [B3]
  //[  ] D6  D5  D3 [  ]

  Now = millis();
  
  // Write Mode LEDs
  if  (Mode == 0) PORTB |= B00100000; else PORTB &= B11011111;
  if ((Mode == 1) || (Mode == 7)) PORTB |= B01000000; else PORTB &= B10111111;    
  if ((Mode == 2) || (Mode == 6)) PORTD |= B10000000; else PORTD &= B01111111;
  if ((Mode == 3) || (Mode == 5)) PORTB |= B10000000; else PORTB &= B01111111;
  if  (Mode == 4) PORTC |= B01000000; else PORTC &= B10111111;

  if (Mode < 4) { // Right Hand Mode
    // Read Input Pins
    Keys   =  (PINF & B10000000) ? 1 : 0;  // .0 
    Keys  |= ((PINB & B00001000) >> 2);    // .1 
    Keys  |= ((PIND & B01000000) >> 4);    // .2
    nThumb =   Keys ^ B00000111;  // Invert Switches
    Keys   =  (PINF & B00010000);          // .4
    Keys  |= ((PINF & B00000011) << 2);    // .3 .2
    Keys  |= ((PINB & B00000001) << 1);    // .1
    Keys  |= ((PINB & B00000010) >> 1);    // .0
    Row[1] =   Keys ^ B00011111;
    Keys   = ((PINF & B00100000) >> 1);    // .4
    Keys  |= ((PINF & B01000000) >> 3);    // .3
    Keys  |= ((PINE & B01000000) >> 4);    // .2
    Keys  |= ((PIND & B00000100) >> 1);    // .1
    Keys  |= ((PINB & B00000100) >> 2);    // .0
    Row[2] =   Keys ^ B00011111;
    Keys   = ((PINB & B00010000) >> 2);    // .2
    Keys  |= ((PIND & B00010000) >> 3);    // .1
    Keys  |= ((PINC & B10000000) >> 7);    // .0
    Row[3] =   Keys ^ B00000111;
    Keys   = ((PIND & B00100000) >> 4);    // .1
    Keys  |= ((PIND & B00001000) >> 3);    // .0
    Row[4] =   Keys ^ B00000011;
  } else {   // Left Hand Mode
    // Read Input Pins
    Keys   =  (PINF & B10000000) ? 2 : 0;  // .0 
    Keys  |= ((PINB & B00001000) >> 3);    // .1 
    nThumb =   Keys ^ B00000011;  // Invert Switches    
    Keys   = ((PINF & B00010000) >> 4);    // .0
    Keys  |=  (PINF & B00000010);          // .1
    Keys  |= ((PINF & B00000001) << 2);    // .2
    Keys  |= ((PINB & B00000011) << 3);    // .3 .4
    Row[1] =   Keys ^ B00011111;
    Keys   = ((PINF & B01100000) >> 5);    // .0 .1
    Keys  |= ((PINE & B01000000) >> 4);    // .2
    Keys  |= ((PIND & B00000100) << 1);    // .3
    Keys  |= ((PINB & B00000100) << 2);    // .4
    Row[2] =   Keys ^ B00011111;
    Keys   = ((PINB & B00010000) >> 4);    // .0
    Keys  |= ((PIND & B00010000) >> 3);    // .1
    Keys  |= ((PINC & B10000000) >> 5);    // .2
    Row[3] =   Keys ^ B00000111;
    Keys   = ((PIND & B01000000) >> 6);    // .0
    Keys  |= ((PIND & B00100000) >> 4);    // .1
//  Keys  |= ((PIND & B00001000) >> 1);    // .2
    Row[4] =   Keys ^ B0000011;
  }
  
  iComboOld = iCombo;
  iLatchOld = iLatch;
  iCombo = (Row[1] | (Row[2] << 5) | (Row[3] << 10) | (Row[4] << 13));

  iLatch |= iCombo;
  if ((iLatch != iLatchOld) || (iCombo != iComboOld)) lBounce = Now;
  if ((Now - lBounce) > 20) {
    if (iLatch != 0) {
      if (Step == 0) {
        iSeq[Step] = iLatch;
        iSendCode  = iLatch;
        Step++;
      } else if ((Step < 7) && (iLatch != iSeq[(Step - 1)])) {
        iSeq[Step] = iLatch;
        iSendCode  = (iLatch ^ iSeq[(Step-1)]);
        Step++;
      } else if (iCombo == 0) {
        iLatch = 0;
        Step   = 0;
      } else if ((Now - lBounce) > 750) {
        lBounce += 75;
        Repeated = true;
        iSendCode = iSeq[(Step-1)];
      }
    }
  }
  if (nThumb != 0) {
    if (nThumb != oThumb) lThumb = Now;
    if ((Now - lThumb) > 30) Thumb |= nThumb;
    if ((nThumb & 4) != 0) ModKey |= 1;
  } 
  oThumb = nThumb;

  if (iSendCode > 0) {
    Send[1] = 0;
    Ptr  = 0;
    QuitLoop = false;
    while ((Send[1] == 0) && (QuitLoop == false)) {
      if (KeyCombo[Ptr] == 255) QuitLoop = true;
      else {
        iCode = KeyCombo[Ptr] | (KeyCombo[(Ptr + 3)] << 5) | (KeyCombo[(Ptr + 6)] << 10) | (KeyCombo[(Ptr + 9)] << 13);
        if (iSendCode == iCode) {
          Offset   = ((Thumb & 3) > 0) ? 2 : 1;
          if ((Mode > 0) && (Mode < 4)) Offset  += (Mode * 3);
          Send[1] = (Offset > 11) ? 0 : KeyCombo[(Ptr + Offset)];
          switch (Step) {
            case 2:
              switch ((char)Send[1]) {
                case (char)KEY_BACKSPACE: if (SendOld == 240) Send[1] = 244; break;  // Mirror
                case (char)KEY_I: if (SendOld == 240) Send[1] = 243; break;
                case (char)KEY_O: if (SendOld == 240) Send[1] = 242; break;                 
                case (char)KEY_E: if (SendOld == (char)KEY_T) Send[1] = KEY_H; break;
                case (char)KEY_A: switch (SendOld) {
                                    case 240:         Send[1] = 241;   break;
                                    case (char)KEY_O: Send[1] = KEY_F; break;
                                  }
                case (char)KEY_S: if (SendOld == (char)KEY_N) Send[1] = KEY_G; break;
              }
              break;
            case 3:
              switch ((char)Send[1]) {
                case (char)KEY_N: if (SendOld == (char)KEY_H) Send[1] = KEY_E; Send[2] = KEY_N; break;
              }
              break;              
          }          
          QuitLoop = true;
        } else Ptr += 12;
      }
    }
    iSendCode = 0;
  }
  
  if (nThumb == 0) {
     if (Send[0] == 0) {
      // +----------------------------------------------------------+  
      // | Generate characters for modifier keys when pressed alone |
      // +----------------------------------------------------------+
      switch (Thumb) {
        case 1: Send[1] = KEY_SPACE; break;
        case 2: Send[1] = KEY_ENTER; break;
      }
    }
    Send[0] = 0;
    Thumb = 0;
  }

  // +-------------------------------------------------------------+  
  // | Convert shifted keys code into base key plus shift modifier |
  // +-------------------------------------------------------------+
  if (Send[1] >= 240) {
    switch (Send[1]) {
      case 240: Mode = (Mode & 4); break;                    // Standard
      case 241: Mode = (Mode & 4) | 1; break;                // Number
      case 242: Mode = (Mode & 4) | 2; break;                // Cursor
      case 243: Mode = (Mode & 4) | 3; break;                // Function
      case 244: Mode ^= 4; break;                            // Switch Hand
      case 251: ModKey = (ModKey ^ 16); break;               // Win
      case 252: ModKey = (ModKey ^ 8); break;                // Alt
      case 253: ModKey = (ModKey ^ 4); break;                // Control
      case 254: ModKey = ((ModKey & 3) < 2) ? 2 : 0; break;  // Shift Lock
      case 255: ModKey = ((ModKey & 3) == 0) ? 1 : 0; break; // Shift
    }
    delay(20);
    Send[0] = Send[1];
    SendOld = Send[1];
    Send[1] = 0;
  }

#ifdef OLED
  if (Mode != oMode) {
    SeeedOled.setTextXY(0,0);            // Set the cursor to Xth Page, Yth Column  
    switch (Mode) {
      case 0: SeeedOled.putString("  RIGHT MODE   "); break;
      case 1: SeeedOled.putString("  Number Mode  "); break;
      case 2: SeeedOled.putString("  Cursor Mode  "); break;
      case 3: SeeedOled.putString(" Function Mode "); break;
      case 4: SeeedOled.putString("  LEFT  MODE   "); break;
      case 5: SeeedOled.putString("  Number Mode  "); break;
      case 6: SeeedOled.putString("  Cursor Mode  "); break;
      case 7: SeeedOled.putString(" Function Mode "); break;
    }
    oMode = Mode;
  }  
  if (oMod != ModKey) {
    SeeedOled.setTextXY(2,9); SeeedOled.putString((ModKey &  3) ? "ON " : "OFF");
    SeeedOled.setTextXY(3,9); SeeedOled.putString((ModKey &  4) ? "ON " : "OFF");
    SeeedOled.setTextXY(4,9); SeeedOled.putString((ModKey &  8) ? "ON " : "OFF");
    SeeedOled.setTextXY(5,9); SeeedOled.putString((ModKey & 16) ? "ON " : "OFF");
    oMod = ModKey;
  }
#endif

  // +-------------------------------------------------+  
  // | Send Characters to computer using USB functions |
  // +-------------------------------------------------+
  if (Send[1] > 0) {
    if (Send[1] < 240) {
      Mod[1]  = 0;
      if (ModKey > 0) {
        if ((ModKey &  3) > 0) Mod[1]  = MODIFIERKEY_SHIFT;
        if ((ModKey &  4) > 0) Mod[1] |= MODIFIERKEY_CTRL;
        if ((ModKey &  8) > 0) Mod[1] |= MODIFIERKEY_ALT;
        if ((ModKey & 16) > 0) Mod[1] |= MODIFIERKEY_GUI;
        ModKey &= B11100010;  // Use up non-latched shift, ctrl & alt      
      }
      if ((Send[1] > 0xA5) && (Send[1] < 0xDF)) {
        switch ((char)Send[1]) {
          case SK_LBRKT:  Send[1] = KEY_9;           Mod[1] = MODIFIERKEY_SHIFT;
                          Send[2] = KEY_0;           Mod[2] = MODIFIERKEY_SHIFT;
                          Send[3] = KEY_LEFT;        Mod[3] = 0;
                          break;
          case SK_LCBRKT: Send[1] = KEY_LEFT_BRACE;  Mod[1] = MODIFIERKEY_SHIFT;
                          Send[2] = KEY_RIGHT_BRACE; Mod[2] = MODIFIERKEY_SHIFT;
                          Send[3] = KEY_LEFT;        Mod[3] = 0;
                          break;
          default:        Send[1] -= 0x90;           Mod[1] = MODIFIERKEY_SHIFT;
                          break;
        }
      } else {
        switch (Send[1]) {
          case (char)KEY_LEFT_BRACE:
                          Send[1] = KEY_LEFT_BRACE;  Mod[1] = 0;
                          Send[2] = KEY_RIGHT_BRACE; Mod[2] = 0;
                          Send[3] = KEY_LEFT;        Mod[3] = 0;
                          break;
        }
      }
      Keyboard.set_modifier(Mod[1]);
      Keyboard.set_key1(Send[1]);
      Keyboard.send_now();
      Send[0] = Send[1];
      SendOld = Send[1];
      Send[1] = 0;
      if (Send[2] != 0) {
        Keyboard.set_modifier(Mod[2]);
        Keyboard.set_key1(Send[2]);
        Keyboard.send_now();
        Send[2] = 0;
        Mod[2]  = 0;
      }
      if (Send[3] != 0) {
        // Send the cursor left with no modifier
        Keyboard.set_modifier(Mod[3]);
        Keyboard.set_key1(Send[3]);
        Keyboard.send_now();
        Send[3] = 0;
        Mod[3]  = 0;
      }
      Keyboard.set_modifier(0);
      Keyboard.set_key1(0);
      Keyboard.send_now();
    }
  }
}

