#include <Wire.h>

//(B5)(B6)(D7)(B7)(C6)
// F4  F1  F0  B0  B1
// F5  F6  E6  D2  B2
//[F7] B4  D4  C7 [B3]
//[  ] D6  D5  D3 [  ]

void setup()
{
    // Data Direction Registers - 0: Input, 1: Output  
  DDRB  = B11100000;
  DDRC  = B01000000;
  DDRD  = B10000000;
  DDRE  = B00000000;
  DDRF  = B00000000;    
  // Data Output Registers - Controls Pull-Up On Inputs & Controls Output
  PORTB = B00011111;  // B0 B1 B2 B3 B4 B6
  PORTC = B10111111;  // C7
  PORTD = B01111111;  // D2 D3 D4 D5 D7
  PORTE = B11111111;  // E6
  PORTF = B11111111;  // F0 F1 F4 F5 F6 F7

  Wire.begin(); // join i2c bus (address optional for master)
}

byte x = 0;
byte Mode = 0;
byte siSend;
byte siRowBit;
byte Row[4];
byte oRow[4];

boolean nBut[18];  
boolean oBut[18];
boolean But[18];
boolean bSendNow;

long Now;
long ButOK[18];
long lSend;

void loop()
{
  if (Mode == 0) PORTB |= B00100000; else PORTB &= B11011111;
  if (Mode == 1) PORTB |= B01000000; else PORTB &= B10111111;    
  if (Mode == 2) PORTD |= B10000000; else PORTD &= B01111111;
  if (Mode == 3) PORTB |= B10000000; else PORTB &= B01111111;
  if (Mode == 4) PORTC |= B01000000; else PORTC &= B10111111;

  Now = millis();

  for (x=0; x<18; x++) oBut[x] = nBut[x];
  // Top Row =====================================
  nBut[0]  = ((PINF & B00010000) == 0);  // F4
  nBut[1]  = ((PINF & B00000010) == 0);  // F1
  nBut[2]  = ((PINF & B00000001) == 0);  // F0
  nBut[3]  = ((PINB & B00000001) == 0);  // B0
  nBut[4]  = ((PINB & B00000010) == 0);  // B1
  // Row 2 =======================================
  nBut[5]  = ((PINF & B00100000) == 0);  // F5
  nBut[6]  = ((PINF & B01000000) == 0);  // F6
  nBut[7]  = ((PINE & B01000000) == 0);  // E6
  nBut[8]  = ((PIND & B00000100) == 0);  // D2
  nBut[9]  = ((PINB & B00000100) == 0);  // B2
  // Row 3 =======================================
  nBut[10] = ((PINB & B00010000) == 0);  // B4
  nBut[11] = ((PIND & B00010000) == 0);  // D4
  nBut[12] = ((PINC & B10000000) == 0);  // C7
  // Row 4 =======================================
  nBut[13] = ((PINF & B10000000) == 0);  // F7
  nBut[14] = ((PIND & B01000000) == 0);  // D6
  nBut[15] = ((PIND & B00100000) == 0);  // D5
  nBut[16] = ((PIND & B00001000) == 0);  // D3
  nBut[17] = ((PINB & B00001000) == 0);  // B3
  
  for (x=0; x<18; x++) {
     if (But[x] != oBut[x]) ButOK[x] = Now;
     else if ((Now - ButOK[x]) > 10) {
       But[x] = nBut[x];
     }
  }

  for (x=0; x<4; x++) oRow[x] = Row[x];
  
  Row[0] = B00100000;
  if (But[0]) Row[0] |=  1;
  if (But[1]) Row[0] |=  2;
  if (But[2]) Row[0] |=  4;
  if (But[3]) Row[0] |=  8;
  if (But[4]) Row[0] |= 16;
  
  Row[1] = B01000000;
  if (But[5]) Row[1] |=  1;
  if (But[6]) Row[1] |=  2;
  if (But[7]) Row[1] |=  4;
  if (But[8]) Row[1] |=  8;
  if (But[9]) Row[1] |= 16;

  Row[2] = B01100000;
  if (But[10]) Row[2] |=  1;
  if (But[11]) Row[2] |=  2;
  if (But[12]) Row[2] |=  4;
  
  Row[3] = B10000000;
  if (But[13]) Row[3] |=  1;
  if (But[14]) Row[3] |=  2;
  if (But[15]) Row[3] |=  4;
  if (But[16]) Row[3] |=  8;
  if (But[17]) Row[3] |= 16;

  bSendNow = 0;
  if ((Now - lSend) > 500) siSend = B00001111;
  else {
    siSend   = 0;
    siRowBit = 1;
    for (x=0; x<4; x++) {
      if (Row[x] != oRow[x]) siSend |= siRowBit;
      siRowBit <<= 1;
    }
  }
  if (siSend > 0) {
    Wire.beginTransmission(4);               // transmit to device #4      
    siRowBit = 1;
    for (x=0; x<4; x++) {
        if ((siSend & siRowBit) != 0) Wire.write(Row[x]);  // sends one byte
        siRowBit <<= 1;
    }
    Wire.endTransmission();                  // stop transmitting
    lSend = Now;
    Mode += 1;
    if (Mode > 4) Mode = 0;
  }
}
