/* Pro Micro Touchpad Code by: Peter Elliot */

#define _clock_pin 1
#define _data_pin  0

#define _col1	15
#define _col2	16
#define _col3	14
#define _col4	8
#define _col5	9

#define _row1	21
#define _row2	20
#define _row3	19
#define _row4 18
#define _row5	4
#define _row6 5
#define _row7	6
#define _row8	7

#define TRUE  1
#define FALSE 0

uint8_t _MaskISR = FALSE;
uint8_t _Mouse   = FALSE;
uint8_t	Matrix[5];

int ReadVal = -1;

uint8_t Seq = 0;

int status = 0;
int xpos   = 128;
int ypos   = 128;
int os,ox,oy;
int8_t dx,dy;

#define pull_low(x)    { pinMode(x, OUTPUT); digitalWrite(x, LOW); }
#define pull_high(x)   pinMode(x, INPUT_PULLUP)
#define bus_idle(x, y) { pinMode(x, INPUT_PULLUP); pinMode(y, INPUT_PULLUP); }
#define wait_CLOCK(x)  while (digitalRead(_clock_pin) != x) {;}

#define ledPin 17
int timer1_counter;
uint8_t	ActiveColumn = 0;

// PS2 Command Set
// ==========================
// 0xe6: Set 1:1 Scaling
// 0xe7: Set 2:1 Scaling
// 0xe8: [#] Sets resolution
// 0xf3: [#] Set sample rate
// 0xea: Set Stream Mode
// 0xf0: Set Non-Stream Mode
// 0xf4: Pause Stream 
// 0xf5: Resume Stream
// 0xff: Reset
// Note: All commands need to Pause/Resume Stream if active

// =================================================
//   The ISR for the Timer1 - Scan Keyboard Matrix
// =================================================
ISR(TIMER1_OVF_vect) {        							// interrupt service routine 
	uint8_t Row;															// Decoded Row Value Variable
	TCNT1 = timer1_counter;   								// preload timer
	Row = (digitalRead(_row1)) ? 1 : 0;
	if (digitalRead(_row2)) Row |= 2;
	if (digitalRead(_row3)) Row |= 4;
	if (digitalRead(_row4)) Row |= 8;
	if (digitalRead(_row5)) Row |= 16;
	if (digitalRead(_row6)) Row |= 32;
	if (digitalRead(_row7)) Row |= 64;
	if (digitalRead(_row8)) Row |= 128;
//	Row =  (PORTF >> 4)												// Calculate Row Value: 21..18 (PF4..PF7)
//			| ((PORTC & B01000000) >> 2)					// + 5 (C6)
//			| ((PORTD & B00010000) << 1)					// + 4 (D4)
//			| ((PORTD & B10000000) >> 1)					// + 9 (D7)
//			| ((PORTE & B01000000) << 1);					// + 7 (E6)
	Matrix[ActiveColumn++] = ~Row;
	if (ActiveColumn > 4) ActiveColumn = 0;
	switch (ActiveColumn) {
	case 0:	pull_high(_col5);	pull_low(_col1); break;	// 15
	case 1:	pull_high(_col1); pull_low(_col2); break;	// 16
	case 2:	pull_high(_col2);	pull_low(_col3); break;	// 14
	case 3:	pull_high(_col3);	pull_low(_col4); break; // 8
	case 4:	pull_high(_col4);	pull_low(_col5); break; // 9
	}
}

// ======================================
//   The ISR for the external interrupt
// ======================================
void ps2int(void) {
	static uint8_t  OldMask  = 1;
	static uint8_t  bitcount = 0;
	static uint8_t  incoming = 0;
	static uint32_t prev_ms  = 0;
	uint32_t now_ms;
	uint8_t n, val;
	int 		value; 
	
	if (!_MaskISR) {
		val    = digitalRead(_data_pin);
		now_ms = millis();
		if ((now_ms - prev_ms > 250) || (OldMask == TRUE)) {
			bitcount = 0;
			incoming = 0;
			Seq 		 = 0;
			OldMask  = FALSE;
		}
		prev_ms = now_ms;
		if ((bitcount > 0) && (bitcount < 9)) incoming |= (val << (bitcount - 1));
		bitcount++;
		if (bitcount == 11) {
			if (_Mouse) {
				ReadVal = -1;
				switch (Seq) {
				case 0:	status = incoming;
								Seq = ((status & 8) != 8) ? 0 : 1;											// .3 should be '1'
								break;
				case 1: dx = (status & 0x10) ? (incoming | 0xff00) : incoming;	// .4 = Negate X
								xpos += dx;
								if (xpos < 0) xpos = 0;
								else if (xpos > 1023) xpos = 1023;
								Seq = 2;																								// Step on in sequence
								break;
				case 2: dy = (status & 0x20) ? (incoming | 0xff00) : incoming;	// .5 = Negate Y
								ypos += dy;
								if (ypos < 0) ypos = 0;
								else if (ypos > 1023) ypos = 1023;
								Seq = 0;																								// Restart sequence
								break;
				}
			} else ReadVal = incoming;
			bitcount = 0;
			incoming = 0;
		}
	} else OldMask = TRUE;
}

void tp_write(int data, int ReplyBytes) {		//
  char i;																		// define loop variable
  char parity = 1;													// Initialize parity value
	_MaskISR = TRUE;													// Mask cursor handling
  bus_idle(_data_pin,_clock_pin);						// Set Bus Idle: Data High,
  delayMicroseconds(300);										// Allow bus to settle
  pull_low(_clock_pin);											// Step 1: Clock Low, Data High
	delayMicroseconds(300);										// Allow bus to settle
  pull_low(_data_pin);											// Step 2: Clock Low, Data Low
  delayMicroseconds(10);										// Allow bus to settle
  pull_high(_clock_pin); 										// Step 3: Clock High, Data Low - Start Bit
	wait_CLOCK(0);											 			// wait for mouse to take control of clock
  for (i = 0; i < 8; i++) {									// Repeat Clock Out Data Byte
    if (data & 0x01) pull_high(_data_pin);	//  Send a '1' or
		else pull_low(_data_pin);								//  Send a '0'
		wait_CLOCK(1);											 		//  wait for clock cycle phase 1
    wait_CLOCK(0);											 		//  wait for clock cycle phase 2
    parity = parity ^ (data & 0x01);				//  Update Parity Bit
    data = data >> 1;												//  Shift data along
  }																					// Until sent
  if (parity) pull_high(_data_pin);					// Send out '1' Parity bit as needed, or
	else pull_low(_data_pin);									//  Send out '0' Parity bit as needed
	wait_CLOCK(1);											 			// wait for clock cycle phase 1
  wait_CLOCK(0);											 			// wait for clock cycle phase 2
	pull_high(_data_pin);											// Clock out stop bit
  delayMicroseconds(50);										// Allow bus to settle
  wait_CLOCK(0);											 			// wait for clock cycle phase 2
  while ((!digitalRead(_clock_pin))					// wait for mouse to switch modes
			|| (!digitalRead(_data_pin))) {;} 		//  wait for mouse to switch modes
  pull_low(_clock_pin); 										// Put a hold on the incoming data.
	if (ReplyBytes > 0) {											// Reply bytes expected
		for (i = 0; i < ReplyBytes; i++) {			//  For Each byte
			tp_read_byte();												//   Read & Discard
		}																				//
	}																					//
	_MaskISR = FALSE;													// Allow cursor handling
}																						//

/* 
int tp_read_byte() {
	ReadVal  = -1;														// Set invalid value
	_Mouse   = FALSE;													// Block Mouse Handling
	_MaskISR = FALSE;													// Allow Byte Handling
  bus_idle(_clock_pin,_data_pin);						// Set Bus Idle: Data High,
	while (ReadVal < 0) {;}										// Wait for byte to clock in...
  pull_low(_clock_pin);											// Put a hold on the incoming data.
	_Mouse = TRUE;														// Mask cursor data
  return ReadVal;														// Return value read
}
*/

int tp_read_byte() {
  int data = 0;															// Initialize data value
	_MaskISR = TRUE;													// Block ISR handling input
  bus_idle(_clock_pin,_data_pin);						// Set Bus Idle: Data High,
	delayMicroseconds(50);										// Allow hardware to settle
  wait_CLOCK(0);											 			// Wait for Start Bit
  delayMicroseconds(5);  										// Debounce
  wait_CLOCK(1);											 			// Eat start bit			
  for (int i = 0; i < 8; i++) {							// For each bit in byte
		bitWrite(data, i, read_bit());					//  Read bit into value
	}																					//  Repeat
  read_bit(); 															// Eat Parity Bit 
  read_bit(); 															// Eat Stop bit should be 1
  pull_low(_clock_pin);											// Put a hold on the incoming data.
  _MaskISR = FALSE;													// Allow ISR to work again
  return data;															// Return value read
}

int read_bit() {														// READ INDIVIDUAL BIT VALUE
  wait_CLOCK(0);											 			// Wait for clock falling edge
  int bit = digitalRead(_data_pin);  				// Read In Bit
  wait_CLOCK(1);											 			// Wait for clock rising edge
  return bit;																// return value
}

void setup() {
	Serial.begin(9600); 											// This pipes to the serial monitor

	pull_high(_row1);													//
	pull_high(_row2);													//
	pull_high(_row3);													//
	pull_high(_row4);													//
	pull_high(_row5);													//
	pull_high(_row6);													//
	pull_high(_row7);													//
	pull_high(_row8);													//
	pull_high(_col1);													//
	pull_high(_col2);													//
	pull_high(_col3);													//
	pull_high(_col4);													//
	pull_high(_col5);													//
	
	// Interrupt Setup - Timer1 
  noInterrupts();           								// disable all interrupts
  TCCR1A = 0;																// Reset Timer1 control register A
  TCCR1B = 0;																// Reset Timer1 control register B
  timer1_counter = 65536 - (62500/25); 			// 34286 - peload timer 65536-16MHz/256/2Hz
  TCNT1 = timer1_counter;   								// preload timer
  TCCR1B |= (1 << CS12);    								// Assign prescaller 256 prescaler 
  TIMSK1 |= (1 << TOIE1);   								// enable timer overflow interrupt
  interrupts();             								// enable all interrupts

	_MaskISR = TRUE;													// Block ISR handler
	bus_idle(_data_pin,_clock_pin);						// Set Bus Idle: Data High,
	attachInterrupt(3, ps2int, FALLING);			// Attach function to Interrupt 3
  delay(20);																// Wait for Idle to take hold...
  tp_write(0xff,1);													// Send Reset to the mouse
  tp_write(0xea,1);													// Send stream mode
  tp_write(0xf4,1); 												// Send enable data reporting
  delay(200);																// Not sure why this needs the delay
	Seq = 0;																	// Reset Sequence variable
	_Mouse   = TRUE;													// Select Mouse Mode
	_MaskISR = FALSE;													// Allow Interrupt Handling
	pull_high(_clock_pin);										// Allow mouse to send...
	Mouse.begin();														// Implement USB Mouse Function
}

void loop() {
	int8_t	loop;
	
	if ((status != os) || (xpos != ox) || (ypos != oy)) {
		if ((dx | dy) != 0) Mouse.move(dx,-dy,0);
		if ((status ^ os) & 1) {
			if (status & 1) {
				Mouse.press();
				pull_low(ledPin);
			} else {
				Mouse.release();
				pull_high(ledPin);
			}
		}
		os = status;
		ox = xpos;
		oy = ypos;
		Serial.print((status & 7));
		Serial.print("x"); Serial.print(xpos);
		Serial.print("y"); Serial.print(ypos);
//		Serial.print(" DDRB:"); Serial.print(DDRB);
//		Serial.print(" PB:"); Serial.println((PORTB & B00111110) >> 1);
		for (loop = 0; loop <5; loop++) {
			Serial.print(">");
			Serial.print(Matrix[loop]);
		}
		Serial.println();
		delay(1);
	}		
	
	//if (digitalRead(_row4)) { pull_high(ledPin); }
	//else { pull_low(ledPin); }
}

